var fs = require('fs');
var Q = require('q');
var path = require('path');
var cheerio = require('cheerio');
var mkdirp = require('mkdirp');
var nunjucks = require('nunjucks');
var decamelize = require('decamelize');

var env = new nunjucks.Environment();
env.addExtension('CodeEditExtension', new CodeEditExtension());

var slidePath;
var slideRootPath;

module.exports = {
    hooks: {
        "init": function(who, guess){
          slidePath = this.config.values.pluginsConfig.revealjs.slidePath || 'Slides';
          slideRootPath = this.config.values.pluginsConfig.revealjs.slideRootPath || '/Slides';
        },
        "page:before": function(page, config) {
            var $ = cheerio.load('<html>' + page.content + '</html>');
            var body = '';
            $('section').each(function(i, section) {
                var newData;
                var render = $(this).data('render') || 'both';

                if (render === 'book'){
                  return;
                }

                if ($(this).html().indexOf('codeeditor') >= 0){
                  var sectionBlock = newData = $(this).text();
                  sectionBlock = processBlock(sectionBlock);
                  $(this).html(sectionBlock);
                }
                else{
                  newData = $(this).text();
                }

                body += ($('<div>').append($(this).clone()).html());

                if (render === 'slide'){
                  $(this).remove();
                }
                else {
                  $(this).replaceWith(newData);
                }
            });


            if (body === ''){
              return page;
            }
            else {
              var rootPath = page.rawPath.replace(page.path, '');
              var framePath = rootPath + slidePath + '/frame.html';
              var revealPath = rootPath + slidePath + page.path.slice(4,-2) + 'html';
              var revealData;
console.log(framePath);
              return Q.nfcall(fs.readFile, framePath, "utf-8")
                .then(function(data) {
                console.log(data);
                  revealData = data.replace(/\%body\%/g, body).replace(/\%root\%/g, slideRootPath);
                  return Q.nfcall(mkdirp, path.dirname(revealPath));
                })
                .then(function(){
                  return Q.nfcall(fs.writeFile, revealPath, revealData, "utf-8");
                })
                .then(function(data2) {
                  var pageData = $('html').html();
                  pageData = pageData.replace(/&apos;/g, "'").replace(/&quot;/g, '"');
                  page.content = pageData;
                  return page;
                })
                .fail(function (error) {
                  console.log("bleh");
                  console.log(error);
                });
            }
        }
    }

};

function processBlock(data){
  return env.renderString(data, { });
}

function CodeEditExtension() {
  this.tags = ['codeeditor'];

  this.parse = function(parser, nodes, lexer) {
    // get the tag token
    var tok = parser.nextToken();

    // parse the args and move after the block end. passing true
    // as the second arg is required if there are no parentheses
    var args = parser.parseSignature(null, true);
    parser.advanceAfterBlockEnd(tok.value);

    // parse the body and possibly the error block, which is optional
    var body = parser.parseUntilBlocks('error', 'endcodeeditor');
    var errorBody = null;

    if(parser.skipSymbol('error')) {
      parser.skip(lexer.TOKEN_BLOCK_END);
      errorBody = parser.parseUntilBlocks('endcodeeditor');
    }

    parser.advanceAfterBlockEnd();

    // See above for notes about CallExtension
    return new nodes.CallExtension(this, 'run', args, [body, errorBody]);
  };

  this.run = function(context, args, body, errorBody) {
    var params = '';
    var newkey;

    for (var key in args){
      if (args.hasOwnProperty(key)) {
        if (key === 'src'){
          newkey = 'codeurl';
        }
        else {
          newkey = decamelize(key, '-');
        }

        if (key !== '__keywords') {
          if (args[key] != '') {
            params += 'data-' + newkey + '="' + args[key] + '" ';
          }
          else{
            params += 'data-' + newkey + ' ';
          }
        }
      }
    }

    var ret = new nunjucks.runtime.SafeString('<iframe  data-code-editor width="100%" height=400 ' + params + '>' + body() + '</iframe>');
    return ret;
  };
}

